#include "StdAfx.h"
#include ".\orsubcondition.h"

IMPLEMENT_SERIAL( OrSubCondition, SubCondition, 1 )


OrSubCondition::OrSubCondition( int col, int comp, double refValue )
{
	orType = true;
	addSubCondition( col, comp, refValue );
	addSubCondition( col, comp, refValue );
	size = 2;
}

OrSubCondition::OrSubCondition( int col, int comp, CString refValueText )
{
	orType = true;
	addSubCondition( col, comp, refValueText );
	size--; // since size-counter starts with 1 (not 0)
}

OrSubCondition::OrSubCondition( OrSubCondition *copyOrSubCondition)
{
	column = copyOrSubCondition->column;
	comparator = copyOrSubCondition->comparator;
	orType = copyOrSubCondition->orType;
	referenceValue = copyOrSubCondition->referenceValue;
	referenceValueText = copyOrSubCondition->referenceValueText;
	size = copyOrSubCondition->size;

	for (int i=0; i < copyOrSubCondition->returnSize(); i++)
	{
		subConditions.Add( new SubCondition( copyOrSubCondition->at( i ) ) );
	}

	textRef = copyOrSubCondition->textRef;
}


OrSubCondition::~OrSubCondition(void)
{
	for (int i=0; i < returnSize(); i++)
	{
		delete at( i );
	}
}

void OrSubCondition::addSubCondition(int column, int comparator, double referenceValue )
{
	subConditions.Add( new SubCondition( column, comparator, referenceValue ) );
	size++;
}

void OrSubCondition::addSubCondition(int column, int comparator, CString referenceValueText )
{
	subConditions.Add( new SubCondition( column, comparator, referenceValueText ) );
	size++;
}

void OrSubCondition::insertSubCondition(int where, int column, int comparator, double referenceValue)
{
	SubCondition *theSubCondition = new SubCondition( column, comparator, referenceValue );
	subConditions.InsertAt( where, theSubCondition );
	size++;
}

void OrSubCondition::insertSubCondition(int where, int column, int comparator, CString referenceValueText)
{
	SubCondition *theSubCondition = new SubCondition( column, comparator, referenceValueText );
	subConditions.InsertAt( where, theSubCondition );
	size++;
}


void OrSubCondition::deleteLastSubCondition()
{
	delete at( returnSize( ) - 1 );
	subConditions.RemoveAt( returnSize( ) - 1 );
	size--;
}

void OrSubCondition::deleteSubCondition( int where )
{
	subConditions.RemoveAt( where );
	size--;
}

// returns subCondNumber when succeeded, else -1
int OrSubCondition::replaceSubCondition(int subCondNumber, int column, int comparator,
									   double referenceValue, int referenceType)
{
	// e.g., size = 5, i.e. max. index = 4
	if (subCondNumber < returnSize( ) )
	{
		at( subCondNumber )->changeValues(column, comparator, referenceValue, referenceType);
		return subCondNumber;
	}

	if (subCondNumber == returnSize() ) // One is added
	{
		addSubCondition( column, comparator, referenceValue );
		return subCondNumber;
	}

	return -1;
}

int OrSubCondition::replaceSubCondition(int subCondNumber, int column, int comparator,
									   CString referenceValueText, int referenceType)
{
	// e.g., size = 5, i.e. max. index = 4
	if (subCondNumber < returnSize() )
	{
		at( subCondNumber )->changeValues(column, comparator, referenceValueText, referenceType);
		return subCondNumber;
	}

	if (subCondNumber == returnSize( ) ) // One is added
	{
		addSubCondition( column, comparator, referenceValueText );
		return subCondNumber;
	}

	return -1;
}

BOOL OrSubCondition::check(WTextFile *theTextFile, int lineNumber)
{
	BOOL result = false;

	for (int i=0; i < returnSize(); i++)
	{
		if (at( i )->check(theTextFile, lineNumber))
			return true;
	}

	return result;
}
int OrSubCondition::returnColumn(int subCondNumber)
{
	return at( subCondNumber )->column;
}

int OrSubCondition::returnComparator(int subCondNumber)
{
	return at( subCondNumber )->comparator;
}

int OrSubCondition::returnReferenceType( int subCondNumber)
{
	return at( subCondNumber )->referenceType;
}

double OrSubCondition::returnReferenceValue(int subCondNumber)
{
	return at( subCondNumber )->referenceValue;
}

CString OrSubCondition::returnReferenceValueText(int subCondNumber)
{
	return at( subCondNumber )->referenceValueText;
}

int OrSubCondition::returnSize( )
{
	return size;
}

SubCondition *OrSubCondition::at( int index )
{
	return subConditions.GetAt( index );
}

void OrSubCondition::Serialize( CArchive& archive )
{
	SubCondition::Serialize( archive ); // base class 'SubCondition' contains size
	
	if( ! archive.IsStoring( ) )
	{
		subConditions.SetSize( size );

		for( int i=0; i < size; i++ )
		{
			subConditions[i] = new SubCondition;
		}
	}
	
	for( int i=0; i < size; i++ )
	{
		subConditions[i]->Serialize( archive );
	}
}

InputError OrSubCondition::checkValidInputs( WTextFile * theTextFile )
{
	InputError inputError;
	
	for ( int i=0; i < returnSize(); i++ )
	{
		inputError = at( i )->checkValidInputs( theTextFile );
		if( inputError.errorCode )
		{
			inputError.orSubCondition = i;
			break;
		}
	}

	return inputError;
}